;Wcterror.asm: software to drive a toilet terror alert board
;This software is licensed under the GNU GPL.
;(c) 2006 Jeroen Domburg

;We're using a tiny2313 here.
.device ATTiny2313
;You might want to include the ATTiny2313-definitions included with
;your assembler. I use gavrasm, which doesn't need them.
;.include "tn2313def.inc"

;The fuses should be configured as follows:
;FUSE_L=0xcf
;FUSE_H=0xdf

;With lights on, the counter increases every 2 seconds. Lights off - the
;counter decreases every 4 seconds. The following equs define the counter
;values at which the light transitions happen.

.equ trans_g2y	=	7	;green to yellow
.equ trans_y2r	=	75	;yellow to red
.equ trans_r2y	=	40	;red to yellow
.equ trans_y2g	=	1	;yellow to green

;Port D
.equ spk1	=	5
.equ led	=	4

;Port B
.equ green	=	0
.equ ldr	=	1
.equ spk2	=	2
.equ yellow	=	3
.equ red	=	6


;Registers
.def dat	=	r0	;Data from an lpm instruction ends up here
.def temp	=	R16 	;The temp register
.def temp2	=	R17 	;Second temp register
.def temp3	=	R18
.def temp4	=	r19
.def temp5	=	r20
.def counter	=	r21
.def zlo	=	r30
.def zhi	=	r31


	rjmp start		; reset vector
	reti			; External Int 0 not enabled
	reti			; External Int 1 not enabled
	reti			; Timer 1 Capture Int not enabled
	reti			; Timer 1 Compare match A Int not enabled
	reti			; Timer 1 Overflow Int not enabled
	reti			; Timer 0 Overflow Int not enabled
	reti			; UART ReceiveInt
	reti			; UART Date Reg Empty Int not enabled
	reti			; UART tx'ed Int Not enabled
	reti			; Anacomp interrupt Not enabled


start:
	ldi temp,RAMEND		; top of memory
	out SPL,temp		; init stack pointer
	ldi temp,$4d		; Port B direction options
	out DDRB,temp		; Setup port direction
	ldi temp,$30		; Port D direction options
	out DDRD,temp		; Setup port direction

	;enable bandgap on a/d converter
	ldi temp,0b01000000
	out acsr,temp

	;Starting at terror level green.
	ldi counter,0
	sbi portb,green

mainloop:
	;Wait 2 secs
	rcall waittwosec
	;Toilet light on?
	sbis acsr,5
	 rjmp lighton

;Light is off
	cbi portd,led
	;decrease counter 2 x as slow as it's increased -> wait some more
	rcall waittwosec
	;Decrease till zero, not under
	dec counter
	cpi counter,$ff
	brne nozero
	ldi counter,0	
nozero:
	;check transitions
	cpi counter,trans_r2y
	breq setyellow
	cpi counter,trans_y2g
	breq setgreen
	rjmp mainloop


;Light is on
lighton:
	sbi portd,led

	;Increase counter, check for max	
	inc counter
	cpi counter,0
	brne notop
	ldi counter,$ff
notop:
	;check transitions
	cpi counter,trans_g2y
	breq setyellow
	cpi counter,trans_y2r
	breq setred

	rjmp mainloop
	
	
;Sub to turn on red light, turn off others. Play ping-sound if this has changed
;the terror level.
setred:
	in temp,portb
	sbi portb,red
	cbi portb,yellow
	cbi portb,green
	sbrs temp,red
	 rcall playping
	rjmp mainloop

;Sub to turn on yellow light, turn off others. Play ping-sound if this has changed
;the terror level.
setyellow:
	in temp,portb
	cbi portb,red
	sbi portb,yellow
	cbi portb,green
	sbrs temp,yellow
	 rcall playping
	rjmp mainloop
	
;Sub to turn on green light, turn off others. Play ping-sound if this has changed
;the terror level.
setgreen:
	in temp,portb
	cbi portb,red
	cbi portb,yellow
	sbi portb,green
	sbrs temp,green
	 rcall playping
	rjmp mainloop

;Waits about two seconds. Not calibrated or anything, but the result
;is close enough.
waittwosec:
	push temp
	push temp2
	push temp3
	ldi temp3,160
waitasec0:
	ldi temp2,0
waitasec1:
	ldi temp,0
waitasec2:
	nop
	dec temp
	brne waitasec2
	dec temp2
	brne waitasec1
	dec temp3
	brne waitasec0
	pop temp3
	pop temp2
	pop temp
	ret


;This basically plays the sound in the sintab lookup table (which
;contains 2 sinuses, one at 4 x the frequency of the others) and
;attenuates the sound in 16 steps by a clever shifting construction
playping:
	;Set up timer 0 as a pwm generator
	ldi temp,0b10110011
	out tccr0a,temp
	ldi temp,0b01
	out tccr0b,temp

	ldi temp3,0
	ldi temp2,0
	ldi temp4,8
sinloop:
	;Get the table value
	ldi zlo,low(sintab*2)
	ldi zhi,high(sintab*2)
	ldi temp,0
	add zlo,temp2
	adc zhi,temp
	lpm
	mov temp,dat
	subi temp,128 ;make it average on 0 so the shift works out allright
	;Shift the value
	push temp3
shloop:
	cpi temp3,2
	brlo shdone
	subi temp3,2
	asr temp
	rjmp shloop
shdone:
	cpi temp3,1
	brne shreallydone
	mov temp3,temp
	asr temp3
	asr temp3
	sub temp,temp3  
shreallydone:
	pop temp3
	subi temp,-128 ;undo the adjustment made earlier

	;output it
	out ocr0a,temp
	out ocr0b,temp
	inc temp2
	andi temp2,31

;wait
	ldi temp,155
	sub temp,temp3
wait:
	nop
	dec temp
	brne wait

	inc temp5
	brne sinloop
	dec temp4
	brne sinloop
	ldi temp4,8
	inc temp3
	cpi temp3,18
	brne sinloop

	;Disable pwm output
	ldi temp,0b0011
	out tccr0a,temp

	ret

;Sound data
sintab:
.db 128,167,194,203,198,191,192,206,227,245,248,230,198,163,138,127,127,128,117,92,57,25,7,10,27,49,63,64,57,52,61,88
